@@ -23,8 +23,13 @@ try:
 except ImportError:
     psycopg2 = None
 
-if sqlite3 is None and psycopg2 is None:
-    raise ImproperlyConfigured('Either sqlite3 or psycopg2 must be installed')
+try:
+    import MySQLdb as mysql
+except ImportError:
+    mysql = None
+
+if sqlite3 is None and psycopg2 is None and mysql is None:
+    raise ImproperlyConfigured('Either sqlite3, psycopg2 or MySQLdb must be installed')
 
 
 DATABASE_NAME = os.environ.get('PEEWEE_DATABASE', 'peewee.db')
@@ -32,6 +37,20 @@ logger = logging.getLogger('peewee.logger')
 
 
 class BaseAdapter(object):
+    """
+    The various subclasses of `BaseAdapter` provide a bridge between the high-
+    level `Database` abstraction and the underlying python libraries like
+    psycopg2.  It also provides a way to unify the pythonic field types with
+    the underlying column types used by the database engine.
+    
+    The `BaseAdapter` provides two types of mappings:    
+    - mapping between filter operations and their database equivalents
+    - mapping between basic field types and their database column types
+    
+    The `BaseAdapter` also is the mechanism used by the `Database` class to:
+    - handle connections with the database
+    - extract information from the database cursor
+    """
     operations = {'eq': '= %s'}
     interpolation = '%s'
     
@@ -60,16 +79,21 @@ class BaseAdapter(object):
         conn.close()
     
     def lookup_cast(self, lookup, value):
+        if lookup in ('contains', 'icontains'):
+            return '%%%s%%' % value
+        elif lookup in ('startswith', 'istartswith'):
+            return '%s%%' % value
         return value
     
     def last_insert_id(self, cursor, model):
-        raise NotImplementedError
+        return cursor.lastrowid
     
     def rows_affected(self, cursor):
         return cursor.rowcount
 
 
 class SqliteAdapter(BaseAdapter):
+    # note the sqlite library uses a non-standard interpolation string
     operations = {
         'lt': '< ?',
         'lte': '<= ?',
@@ -89,9 +113,6 @@ class SqliteAdapter(BaseAdapter):
     def connect(self, database, **kwargs):
         return sqlite3.connect(database, **kwargs)
     
-    def last_insert_id(self, cursor, model):
-        return cursor.lastrowid
-    
     def lookup_cast(self, lookup, value):
         if lookup == 'contains':
             return '*%s*' % value
@@ -134,15 +155,43 @@ class PostgresqlAdapter(BaseAdapter):
             model._meta.db_table, model._meta.pk_name))
         return cursor.fetchone()[0]
     
-    def lookup_cast(self, lookup, value):
-        if lookup in ('contains', 'icontains'):
-            return '%%%s%%' % value
-        elif lookup in ('startswith', 'istartswith'):
-            return '%s%%' % value
-        return value
+
+class MySQLAdapter(BaseAdapter):
+    operations = {
+        'lt': '< %s',
+        'lte': '<= %s',
+        'gt': '> %s',
+        'gte': '>= %s',
+        'eq': '= %s',
+        'ne': '!= %s', # watch yourself with this one
+        'in': 'IN (%s)', # special-case to list q-marks
+        'is': 'IS %s',
+        'icontains': 'LIKE %s', # surround param with %'s
+        'contains': 'LIKE BINARY %s', # surround param with *'s
+        'istartswith': 'LIKE %s',
+        'startswith': 'LIKE BINARY %s',
+    }
+
+    def connect(self, database, **kwargs):
+        return mysql.connect(db=database, **kwargs)
+
+    def get_field_overrides(self):
+        return {
+            'primary_key': 'integer AUTO_INCREMENT',
+            'boolean': 'bool',
+            'float': 'double precision',
+            'text': 'longtext',
+        }
 
 
 class Database(object):
+    """
+    A high-level api for working with the supported database engines.  `Database`
+    provides a wrapper around some of the functions performed by the `Adapter`,
+    in addition providing support for:
+    - execution of SQL queries
+    - creating and dropping tables and indexes
+    """
     def __init__(self, adapter, database, **connect_kwargs):
         self.adapter = adapter
         self.database = database
@@ -208,12 +257,43 @@ class Database(object):
     def drop_table(self, model_class, fail_silently=False):
         framing = fail_silently and 'DROP TABLE IF EXISTS %s;' or 'DROP TABLE %s;'
         self.execute(framing % model_class._meta.db_table, commit=True)
+    
+    def get_indexes_for_table(self, table):
+        raise NotImplementedError
 
 
-database = Database(SqliteAdapter(), DATABASE_NAME)
+class SqliteDatabase(Database):
+    def __init__(self, database, **connect_kwargs):
+        super(SqliteDatabase, self).__init__(SqliteAdapter(), database, **connect_kwargs)
+    
+    def get_indexes_for_table(self, table):
+        res = self.execute('PRAGMA index_list(%s);' % table)
+        rows = sorted([(r[1], r[2] == 1) for r in res.fetchall()])
+        return rows
+
+
+class PostgresqlDatabase(Database):
+    def __init__(self, database, **connect_kwargs):
+        super(PostgresqlDatabase, self).__init__(PostgresqlAdapter(), database, **connect_kwargs)
+
+
+class MySQLDatabase(Database):
+    def __init__(self, database, **connect_kwargs):
+        super(MySQLDatabase, self).__init__(MySQLAdapter(), database, **connect_kwargs)
+    
+    def get_indexes_for_table(self, table):
+        res = self.execute('SHOW INDEXES IN %s;' % table)
+        rows = sorted([(r[2], r[1] == 0) for r in res.fetchall()])
+        return rows
 
 
 class QueryResultWrapper(object):
+    """
+    Provides an iterator over the results of a raw Query, additionally doing
+    two things:
+    - converts rows from the database into model instances
+    - ensures that multiple iterations do not result in multiple queries
+    """
     def __init__(self, model, cursor):
         self.model = model
         self.cursor = cursor
@@ -252,13 +332,14 @@ class QueryResultWrapper(object):
             raise StopIteration
 
 
+# semantic wrappers for ordering the results of a `SelectQuery`
 def asc(f):
     return (f, 'ASC')
 
 def desc(f):
     return (f, 'DESC')
 
-# select wrappers
+# wrappers for performing aggregation in a `SelectQuery`
 def Count(f, alias='count'):
     return ('COUNT', f, alias)
 
@@ -268,6 +349,8 @@ def Max(f, alias='max'):
 def Min(f, alias='min'):
     return ('MIN', f, alias)
 
+# decorator for query methods to indicate that they change the state of the
+# underlying data structures
 def mark_query_dirty(func):
     def inner(self, *args, **kwargs):
         self._dirty = True
@@ -374,6 +457,10 @@ def parseq(*args, **kwargs):
     return node
 
 
+class EmptyResultException(Exception):
+    pass
+
+
 class BaseQuery(object):
     query_separator = '__'
     requires_commit = True
@@ -415,6 +502,8 @@ class BaseQuery(object):
                     lookup_value = rhs
                     operation = 'IN (%s)'
                 else:
+                    if not rhs:
+                        raise EmptyResultException
                     lookup_value = [field.db_value(o) for o in rhs]
                     operation = self.operations[op] % \
                         (','.join([self.interpolation for v in lookup_value]))
@@ -523,6 +612,15 @@ class BaseQuery(object):
         
         return computed_joins, where_with_alias, where_data, alias_map
     
+    def convert_where_to_params(self, where_data):
+        flattened = []
+        for clause in where_data:
+            if isinstance(clause, (tuple, list)):
+                flattened.extend(clause)
+            else:
+                flattened.append(clause)
+        return flattened
+    
     def parse_node(self, node, model, alias_map):
         query = []
         query_data = []
@@ -581,6 +679,22 @@ class BaseQuery(object):
         return self.database.execute(query, params, self.requires_commit)
 
 
+class RawQuery(BaseQuery):
+    def __init__(self, model, query, *params):
+        self._sql = query
+        self._params = list(params)
+        super(RawQuery, self).__init__(model)
+    
+    def sql(self):
+        return self._sql, self._params
+    
+    def execute(self):
+        return QueryResultWrapper(self.model, self.raw_execute())
+    
+    def __iter__(self):
+        return self.execute()
+
+
 class SelectQuery(BaseQuery):
     requires_commit = False
     
@@ -610,8 +724,7 @@ class SelectQuery(BaseQuery):
         else:
             self.query = 'COUNT(%s)' % (self.model._meta.pk_name)
         
-        db = self.model._meta.database
-        res = db.execute(*self.sql())
+        res = self.database.execute(*self.sql())
         
         self.query = tmp_query
         self._pagination = tmp_pagination
@@ -710,11 +823,8 @@ class SelectQuery(BaseQuery):
             pieces.append(joins)
         if where:
             pieces.append('WHERE %s' % where)
-            for clause in where_data:
-                if isinstance(clause, (tuple, list)):
-                    params.extend(clause)
-                else:
-                    params.append(clause)
+            params.extend(self.convert_where_to_params(where_data))
+        
         if group_by:
             pieces.append('GROUP BY %s' % group_by)
         if having:
@@ -731,9 +841,12 @@ class SelectQuery(BaseQuery):
     
     def execute(self):
         if self._dirty:
-            self._qr = QueryResultWrapper(self.model, self.raw_execute())
-            self._dirty = False
-            return self._qr
+            try:
+                self._qr = QueryResultWrapper(self.model, self.raw_execute())
+                self._dirty = False
+                return self._qr
+            except EmptyResultException:
+                return iter([])
         else:
             # call the __iter__ method directly
             return iter(self._qr)
@@ -750,7 +863,13 @@ class UpdateQuery(BaseQuery):
     def parse_update(self):
         sets = {}
         for k, v in self.update_query.iteritems():
-            field = self.model._meta.get_field_by_name(k)
+            try:
+                field = self.model._meta.get_field_by_name(k)
+            except AttributeError:
+                field = self.model._meta.get_related_field_by_name(k)
+                if field is None:
+                    raise
+            
             sets[field.name] = field.db_value(v)
         
         return sets
@@ -774,7 +893,7 @@ class UpdateQuery(BaseQuery):
         
         if where:
             pieces.append('WHERE %s' % where)
-            params.extend(where_data)
+            params.extend(self.convert_where_to_params(where_data))
         
         return ' '.join(pieces), params
     
@@ -799,7 +918,7 @@ class DeleteQuery(BaseQuery):
         
         if where:
             pieces.append('WHERE %s' % where)
-            params.extend(where_data)
+            params.extend(self.convert_where_to_params(where_data))
         
         return ' '.join(pieces), params
     
@@ -850,6 +969,7 @@ class InsertQuery(BaseQuery):
 
 class Field(object):
     db_field = ''
+    default = None
     field_template = "%(column_type)s%(nullable)s"
 
     def get_attributes(self):
@@ -859,6 +979,7 @@ class Field(object):
         self.null = null
         self.db_index = db_index
         self.attributes = self.get_attributes()
+        self.default = kwargs.get('default', None)
         
         kwargs['nullable'] = ternary(self.null, '', ' NOT NULL')
         self.attributes.update(kwargs)
@@ -875,7 +996,7 @@ class Field(object):
     
     def to_sql(self):
         rendered = self.render_field_template()
-        return '"%s" %s' % (self.name, rendered)
+        return '%s %s' % (self.name, rendered)
     
     def null_wrapper(self, value, default=None):
         if (self.null and value is None) or default is None:
@@ -933,9 +1054,10 @@ class DateTimeField(Field):
     db_field = 'datetime'
     
     def python_value(self, value):
-        if value is not None:
+        if isinstance(value, basestring):
             value = value.rsplit('.', 1)[0]
             return datetime(*time.strptime(value, '%Y-%m-%d %H:%M:%S')[:6])
+        return value
 
 
 class IntegerField(Field):
@@ -1009,7 +1131,7 @@ class ReverseForeignRelatedObject(object):
 
 class ForeignKeyField(IntegerField):
     db_field = 'foreign_key'
-    field_template = '%(column_type)s%(nullable)s REFERENCES "%(to_table)s" ("%(to_pk)s")'
+    field_template = '%(column_type)s%(nullable)s REFERENCES %(to_table)s (%(to_pk)s)'
     
     def __init__(self, to, null=False, related_name=None, *args, **kwargs):
         self.to = to
@@ -1039,17 +1161,27 @@ class ForeignKeyField(IntegerField):
         if isinstance(value, Model):
             return value.get_pk()
         return value or None
+    
+    def db_value(self, value):
+        if isinstance(value, Model):
+            return value.get_pk()
+        return value
+
+
+# define a default database object in the module scope
+database = SqliteDatabase(DATABASE_NAME)
 
 
 class BaseModelOptions(object):
     def __init__(self, model_class, options=None):
+        # configurable options
+        options = options or {'database': database}
+        for k, v in options.items():
+            setattr(self, k, v)
+        
         self.rel_fields = {}
         self.fields = {}
         self.model_class = model_class
-        
-        # configurable options
-        options = options or {}
-        self.database = options.get('database', database)
     
     def get_field_by_name(self, name):
         if name in self.fields:
@@ -1078,6 +1210,8 @@ class BaseModelOptions(object):
 
 
 class BaseModel(type):
+    inheritable_options = ['database']
+    
     def __new__(cls, name, bases, attrs):
         cls = super(BaseModel, cls).__new__(cls, name, bases, attrs)
 
@@ -1092,7 +1226,7 @@ class BaseModel(type):
                 continue
             
             for (k, v) in base_meta.__dict__.items():
-                if not k.startswith('_') and k not in attr_dict:
+                if k in cls.inheritable_options and k not in attr_dict:
                     attr_dict[k] = v
         
         _meta = BaseModelOptions(cls, attr_dict)
@@ -1138,8 +1272,17 @@ class Model(object):
                other.get_pk() == self.get_pk()
     
     def get_field_dict(self):
-        field_val = lambda f: (f.name, getattr(self, f.name))
-        pairs = map(field_val, self._meta.fields.values())
+        def get_field_val(field):
+            field_value = getattr(self, field.name)
+            if not self.get_pk() and field_value is None and field.default is not None:
+                if callable(field.default):
+                    field_value = field.default()
+                else:
+                    field_value = field.default
+                setattr(self, field.name, field_value)
+            return (field.name, field_value)
+        
+        pairs = map(get_field_val, self._meta.fields.values())
         return dict(pairs)
     
     @classmethod
@@ -1173,6 +1316,10 @@ class Model(object):
     @classmethod
     def delete(cls, **query):
         return DeleteQuery(cls, **query)
+    
+    @classmethod
+    def raw(cls, sql, *params):
+        return RawQuery(cls, sql, *params)
 
     @classmethod
     def create(cls, **query):
